#!/usr/bin/env python3
"""
SPS Search
by Javantea
July 27, 2022

Find seeds with the lowest max drought length in the first 300 pieces.
This is very slow, so use pypy3 or run multithreaded.
3m 36s on pypy3 with 32 threads.

Seeds 0-N is omitted because there are many seeds that repeat every 7 pieces.
"""
import tetris_sps
import multiprocessing


def getMaxDroughtLength(args):
    seed, num_pieces = args
    sps = tetris_sps.ThreadSafeSPS(seed)
    v = [tetris_sps.lookup[sps.pickTetriminoSeed()] for i in range(num_pieces)]
    droughts = [len(x) for x in (''.join(v)).split('I')]
    return (seed, max(droughts))

def main():
    min_i = 1000
    num_pieces = 300
    pool = multiprocessing.Pool()
    res = pool.imap(getMaxDroughtLength, [(x, 300) for x in range(1, 0x1000000)], chunksize=1000)
    for seed, num_i in res:
        if num_i < min_i:
            if num_i == 7: continue
            best_seed = seed
            min_i = num_i
            print('{0:06x} {1} max drought out of 300 pieces'.format(best_seed, min_i))

if __name__ == '__main__':
    main()
