#!/usr/bin/env python3
"""
SPS Search
by Javantea
July 27, 2022

Find seeds with the most number of I pieces in the first 300 pieces.
This is very slow, so use pypy3 or run multithreaded.
3m 36s on pypy3 with 32 threads.
"""
import tetris_sps
import multiprocessing


def getSequenceI(args):
    seed, num_pieces = args
    sps = tetris_sps.ThreadSafeSPS(seed)
    v = [tetris_sps.lookup[sps.pickTetriminoSeed()] for i in range(num_pieces)]
    num_i = len([x for x in v if x == 'I'])
    return (seed, num_i)

def main():
    min_i = 1000
    num_pieces = 300
    pool = multiprocessing.Pool()
    res = pool.imap(getSequenceI, [(x, 300) for x in range(0x1000000)], chunksize=1000)
    for seed, num_i in res:
        if num_i < min_i:
            best_seed = seed
            min_i = num_i
            print('{0:06x} {1} I pieces out of 300'.format(best_seed, min_i))

if __name__ == '__main__':
    main()
